@@ -10,68 +10,6 @@ public-API constructor macros.
 
 #![doc(hidden)]
 
-/** Ensures that the ordering tokens map to a known ordering type path.
-
-Note: non-`const` constructor expressions cannot be used to initialize `static`
-bindings. Unfortunately, replacing the `from_slice` calls with literal
-construction of the pointer representation, and type-casting it into the correct
-type, is *also* unstable, as it requires dereferencing a raw pointer inside a
-`static` context.
-**/
-#[doc(hidden)]
-#[macro_export]
-macro_rules! __bits_from_slice {
-	(mut LocalBits, $len:expr, $slice:ident) => {
-		$crate::slice::BitSlice::<$crate::order::LocalBits, _>::from_slice_mut(
-			&mut $slice,
-			)
-		.expect("slice construction exceeded capacity")
-		.get_unchecked_mut(.. $len)
-	};
-	(mut Lsb0, $len:expr, $slice:ident) => {
-		$crate::slice::BitSlice::<$crate::order::Lsb0, _>::from_slice_mut(
-			&mut $slice,
-			)
-		.expect("slice construction exceeded capacity")
-		.get_unchecked_mut(.. $len)
-	};
-	(mut Msb0, $len:expr, $slice:ident) => {
-		$crate::slice::BitSlice::<$crate::order::Msb0, _>::from_slice_mut(
-			&mut $slice,
-			)
-		.expect("slice construction exceeded capacity")
-		.get_unchecked_mut(.. $len)
-	};
-	(mut $order:tt, $len:expr, $slice:ident) => {
-		$crate::slice::BitSlice::<$order, _>::from_slice_mut(&mut $slice)
-			.expect("slice construction exceeded capacity")
-			.get_unchecked_mut(.. $len)
-	};
-
-	(LocalBits, $len:expr, $slice:ident) => {
-		$crate::slice::BitSlice::<$crate::order::LocalBits, _>::from_slice(
-			&$slice,
-			)
-		.expect("slice construction exceeded capacity")
-		.get_unchecked(.. $len)
-	};
-	(Lsb0, $len:expr, $slice:ident) => {
-		$crate::slice::BitSlice::<$crate::order::Lsb0, _>::from_slice(&$slice)
-			.expect("slice construction exceeded capacity")
-			.get_unchecked(.. $len)
-	};
-	(Msb0, $len:expr, $slice:ident) => {
-		$crate::slice::BitSlice::<$crate::order::Msb0, _>::from_slice(&$slice)
-			.expect("slice construction exceeded capacity")
-			.get_unchecked(.. $len)
-	};
-	($order:tt, $len:expr, $slice:ident) => {
-		$crate::slice::BitSlice::<$order, _>::from_slice(&$slice)
-			.expect("slice construction exceeded capacity")
-			.get_unchecked(.. $len)
-	};
-}
-
 /** Accumulates a stream of bit expressions into a compacted array of elements.
 
 This macro constructs a well-ordered `[T; N]` array expression usable in `const`
@@ -240,7 +178,10 @@ macro_rules! __elt_from_bits {
 	//  Known orderings can be performed immediately.
 	(
 		Lsb0, $store:ident;
-		$($a:tt, $b:tt, $c:tt, $d:tt, $e:tt, $f:tt, $g:tt, $h:tt),*
+		$(
+			$a:expr, $b:expr, $c:expr, $d:expr,
+			$e:expr, $f:expr, $g:expr, $h:expr
+		),*
 	) => {
 		$crate::__ty_from_bytes!(
 			Lsb0, $store, [$($crate::macros::internal::u8_from_le_bits(
@@ -251,7 +192,10 @@ macro_rules! __elt_from_bits {
 	};
 	(
 		Msb0, $store:ident;
-		$($a:tt, $b:tt, $c:tt, $d:tt, $e:tt, $f:tt, $g:tt, $h:tt),*
+		$(
+			$a:expr, $b:expr, $c:expr, $d:expr,
+			$e:expr, $f:expr, $g:expr, $h:expr
+		),*
 	) => {
 		$crate::__ty_from_bytes!(
 			Msb0, $store, [$($crate::macros::internal::u8_from_be_bits(
@@ -262,7 +206,10 @@ macro_rules! __elt_from_bits {
 	};
 	(
 		LocalBits, $store:ident;
-		$($a:tt, $b:tt, $c:tt, $d:tt, $e:tt, $f:tt, $g:tt, $h:tt),*
+		$(
+			$a:expr, $b:expr, $c:expr, $d:expr,
+			$e:expr, $f:expr, $g:expr, $h:expr
+		),*
 	) => {
 		$crate::__ty_from_bytes!(
 			LocalBits, $store, [$($crate::macros::internal::u8_from_ne_bits(
@@ -272,43 +219,25 @@ macro_rules! __elt_from_bits {
 		)
 	};
 
-	//  Unknown orders are currently unsupported in `macro_rules!`.
 	(
 		$order:tt, $store:ident;
-		$($a:tt, $b:tt, $c:tt, $d:tt, $e:tt, $f:tt, $g:tt, $h:tt),*
+		$(
+			$a:expr, $b:expr, $c:expr, $d:expr,
+			$e:expr, $f:expr, $g:expr, $h:expr
+		),*
 	) => {{
-		/* Note: they can *become* supported, by adding an `ident <-` argument
-		to the constructor macros that allows construction of runtime non-const
-		values into a binding with a constant initializer. At present, this is
-		not done because the `bitarr!` macro relies on lifetime extension of a
-		borrowed temporary in order to correctly produce a stack object usable
-		even by the `bits!` borrowing constructor. As of this commit, the
-		compiler appears to fail to properly extend the lifetime of a terminal
-		expression of a block, but does extend the lifetime of a free
-		expression. This prevents using a block to construct temporary
-		initializers before collecting them into a `BitArray` which can be
-		lifetime-extended by the caller.
-
-		While these two goals (stack-allocation of macro-constructed buffers, vs
-		accepting any ordering type in the macros) are in contention, stack
-		allocation is going to win. This will be revisited if the compiler
-		improves its lifetime-extension behavior.
-		*/
-		compile_error!("The ordering argument you provided is unrecognized, and as such cannot be used in const-initializers.");
 		let mut tmp: $store = 0;
+		let _tmp_bits = BitSlice::<$order, $store>::from_element_mut(&mut tmp);
 		let mut _idx = 0;
 		$(
-			tmp = tmp
-			| if $a != 0 { $order::select(unsafe { BitIdx::new_unchecked(_idx + 0) }).value() } else { 0 }
-			| if $b != 0 { $order::select(unsafe { BitIdx::new_unchecked(_idx + 1) }).value() } else { 0 }
-			| if $c != 0 { $order::select(unsafe { BitIdx::new_unchecked(_idx + 2) }).value() } else { 0 }
-			| if $d != 0 { $order::select(unsafe { BitIdx::new_unchecked(_idx + 3) }).value() } else { 0 }
-			| if $e != 0 { $order::select(unsafe { BitIdx::new_unchecked(_idx + 4) }).value() } else { 0 }
-			| if $f != 0 { $order::select(unsafe { BitIdx::new_unchecked(_idx + 5) }).value() } else { 0 }
-			| if $g != 0 { $order::select(unsafe { BitIdx::new_unchecked(_idx + 6) }).value() } else { 0 }
-			| if $h != 0 { $order::select(unsafe { BitIdx::new_unchecked(_idx + 7) }).value() } else { 0 }
-			;
-			_idx += 8;
+			_tmp_bits.set(_idx, $a != 0); _idx += 1;
+			_tmp_bits.set(_idx, $b != 0); _idx += 1;
+			_tmp_bits.set(_idx, $c != 0); _idx += 1;
+			_tmp_bits.set(_idx, $d != 0); _idx += 1;
+			_tmp_bits.set(_idx, $e != 0); _idx += 1;
+			_tmp_bits.set(_idx, $f != 0); _idx += 1;
+			_tmp_bits.set(_idx, $g != 0); _idx += 1;
+			_tmp_bits.set(_idx, $h != 0); _idx += 1;
 		)*
 		tmp
 	}};
@@ -386,8 +315,7 @@ pub const fn u8_from_le_bits(
 	f: bool,
 	g: bool,
 	h: bool,
-) -> u8
-{
+) -> u8 {
 	(a as u8)
 		| ((b as u8) << 1)
 		| ((c as u8) << 2)
@@ -410,8 +338,7 @@ pub const fn u8_from_be_bits(
 	f: bool,
 	g: bool,
 	h: bool,
-) -> u8
-{
+) -> u8 {
 	(h as u8)
 		| ((g as u8) << 1)
 		| ((f as u8) << 2)
