@@ -1545,11 +1545,7 @@ where
 	/// [`split_at`]: #method.split_at
 	#[inline]
 	pub unsafe fn split_at_unchecked(&self, mid: usize) -> (&Self, &Self) {
-		match mid {
-			0 => (Self::empty(), self),
-			n if n == self.len() => (self, Self::empty()),
-			_ => (self.get_unchecked(.. mid), self.get_unchecked(mid ..)),
-		}
+		(self.get_unchecked(.. mid), self.get_unchecked(mid ..))
 	}
 
 	/// Splits a mutable slice at some mid-point, without checking boundary
@@ -1597,19 +1593,12 @@ where
 	pub unsafe fn split_at_unchecked_mut(
 		&mut self,
 		mid: usize,
-	) -> (&mut BitSlice<O, T::Alias>, &mut BitSlice<O, T::Alias>)
-	{
+	) -> (&mut BitSlice<O, T::Alias>, &mut BitSlice<O, T::Alias>) {
 		let bp = self.alias_mut().bitptr();
-		match mid {
-			0 => (BitSlice::empty_mut(), bp.to_bitslice_mut()),
-			n if n == self.len() => {
-				(bp.to_bitslice_mut(), BitSlice::empty_mut())
-			},
-			_ => (
-				bp.to_bitslice_mut().get_unchecked_mut(.. mid),
-				bp.to_bitslice_mut().get_unchecked_mut(mid ..),
-			),
-		}
+		(
+			bp.to_bitslice_mut().get_unchecked_mut(.. mid),
+			bp.to_bitslice_mut().get_unchecked_mut(mid ..),
+		)
 	}
 
 	/// Splits a mutable slice at some mid-point, without checking boundary
@@ -1629,8 +1618,7 @@ where
 	pub(crate) unsafe fn split_at_unchecked_mut_noalias(
 		&mut self,
 		mid: usize,
-	) -> (&mut Self, &mut Self)
-	{
+	) -> (&mut Self, &mut Self) {
 		//  Split the slice at the requested midpoint, adding an alias layer
 		let (head, tail) = self.split_at_unchecked_mut(mid);
 		//  Remove the new alias layer.
@@ -1945,8 +1933,7 @@ where
 	pub fn split_at_aliased_mut(
 		&mut self,
 		mid: usize,
-	) -> (&mut Self, &mut Self)
-	{
+	) -> (&mut Self, &mut Self) {
 		let (head, tail) = self.split_at_mut(mid);
 		unsafe { (Self::unalias_mut(head), Self::unalias_mut(tail)) }
 	}
